@@ -13,12 +13,13 @@
 ##############################################################################
 """Component Architecture Tests
 
-$Id: tests.py 97939 2009-03-12 00:01:36Z nadako $
+$Id: tests.py 100192 2009-05-21 13:22:14Z faassen $
 """
 import re
 import unittest
 import transaction
 import persistent
+from cStringIO import StringIO
 
 from zope import interface, component
 from zope.interface.verify import verifyObject
@@ -28,10 +29,19 @@ from zope.testing import doctest, renormalizing
 from zope.component.interfaces import ComponentLookupError
 from zope.component.interfaces import IComponentArchitecture
 from zope.component.interfaces import IComponentLookup
-from zope.component.testing import setUp, tearDown
+from zope.component.testing import setUp, tearDown, PlacelessSetup
 import zope.component.persistentregistry
 import zope.component.globalregistry
 
+from zope.configuration.xmlconfig import XMLConfig, xmlconfig
+from zope.configuration.exceptions import ConfigurationError
+from zope.security.checker import ProxyFactory
+
+from zope.component.testfiles.adapter import A1, A2, A3
+from zope.component.testfiles.components import IContent, Content
+from zope.component.testfiles.components import IApp
+from zope.component.testfiles.views import Request, IC, IV, V1, R1, IR
+
 # side effect gets component-based event dispatcher installed.
 # we should obviously make this more explicit
 import zope.component.event
@@ -1042,7 +1052,6 @@ class StandaloneTests(unittest.TestCase):
         import subprocess
         import sys
         import os
-        import StringIO
         import tempfile
         import pickle
 
@@ -1188,6 +1197,402 @@ class HookableTests(unittest.TestCase):
         else:
             self.fail('Deleted implementation')
 
+class Ob3(object):
+    interface.implements(IC)
+
+template = """<configure
+   xmlns='http://namespaces.zope.org/zope'
+   i18n_domain='zope'>
+   %s
+   </configure>"""
+
+
+class ResourceViewTests(PlacelessSetup, unittest.TestCase):
+
+    def setUp(self):
+        super(ResourceViewTests, self).setUp()
+        XMLConfig('meta.zcml', zope.component)()
+        XMLConfig('meta.zcml', zope.security)()
+
+    def testView(self):
+        ob = Ob3()
+        request = Request(IV)
+        self.assertEqual(
+            zope.component.queryMultiAdapter((ob, request), name=u'test'), None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"/>
+            '''
+            ))
+
+        self.assertEqual(
+            zope.component.queryMultiAdapter((ob, request),
+                                             name=u'test').__class__,
+            V1)
+
+
+    def testMultiView(self):
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.adapter.A3"
+                  for="zope.component.testfiles.views.IC
+                       zope.component.testfiles.adapter.I1
+                       zope.component.testfiles.adapter.I2"
+                  type="zope.component.testfiles.views.IV"/>
+            '''
+            ))
+
+
+        ob = Ob3()
+        a1 = A1()
+        a2 = A2()
+        request = Request(IV)
+        view = zope.component.queryMultiAdapter((ob, a1, a2, request),
+                                                name=u'test')
+        self.assertEqual(view.__class__, A3)
+        self.assertEqual(view.context, (ob, a1, a2, request))
+
+
+    def testMultiView_fails_w_multiple_factories(self):
+        self.assertRaises(
+            ConfigurationError,
+            xmlconfig,
+            StringIO(template %
+              '''
+              <view name="test"
+                    factory="zope.component.testfiles.adapter.A3
+                             zope.component.testfiles.adapter.A2"
+                    for="zope.component.testfiles.views.IC
+                         zope.component.testfiles.adapter.I1
+                         zope.component.testfiles.adapter.I2"
+                    type="zope.component.testfiles.views.IV"/>
+              '''
+              )
+            )
+
+    def testView_w_multiple_factories(self):
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.adapter.A1
+                           zope.component.testfiles.adapter.A2
+                           zope.component.testfiles.adapter.A3
+                           zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"/>
+            '''
+            ))
+
+        ob = Ob3()
+
+        # The view should be a V1 around an A3, around an A2, around
+        # an A1, anround ob:
+        view = zope.component.queryMultiAdapter((ob, Request(IV)), name=u'test')
+        self.assertEqual(view.__class__, V1)
+        a3 = view.context
+        self.assertEqual(a3.__class__, A3)
+        a2 = a3.context[0]
+        self.assertEqual(a2.__class__, A2)
+        a1 = a2.context[0]
+        self.assertEqual(a1.__class__, A1)
+        self.assertEqual(a1.context[0], ob)
+
+    def testView_fails_w_no_factories(self):
+        self.assertRaises(ConfigurationError,
+                          xmlconfig,
+                          StringIO(template %
+                                   '''
+                                   <view name="test"
+                                   factory=""
+                                   for="zope.component.testfiles.views.IC"
+                                   type="zope.component.testfiles.views.IV"/>
+                                   '''
+                                   ),
+                          )
+
+
+    def testViewThatProvidesAnInterface(self):
+        ob = Ob3()
+        self.assertEqual(
+            zope.component.queryMultiAdapter((ob, Request(IR)), IV, u'test'),
+            None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IR"
+                  />
+            '''
+            ))
+
+        self.assertEqual(
+            zope.component.queryMultiAdapter((ob, Request(IR)), IV, u'test'),
+            None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IR"
+                  provides="zope.component.testfiles.views.IV"
+                  />
+            '''
+            ))
+
+        v = zope.component.queryMultiAdapter((ob, Request(IR)), IV, u'test')
+        self.assertEqual(v.__class__, V1)
+
+
+    def testUnnamedViewThatProvidesAnInterface(self):
+        ob = Ob3()
+        self.assertEqual(
+            zope.component.queryMultiAdapter((ob, Request(IR)), IV), None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <view factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IR"
+                  />
+            '''
+            ))
+
+        v = zope.component.queryMultiAdapter((ob, Request(IR)), IV)
+        self.assertEqual(v, None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <view factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IR"
+                  provides="zope.component.testfiles.views.IV"
+                  />
+            '''
+            ))
+
+        v = zope.component.queryMultiAdapter((ob, Request(IR)), IV)
+        self.assertEqual(v.__class__, V1)
+
+    def testViewHavingARequiredClass(self):
+        xmlconfig(StringIO(template % (
+            '''
+            <view
+              for="zope.component.testfiles.components.Content"
+              type="zope.component.testfiles.views.IR"
+              factory="zope.component.testfiles.adapter.A1"
+              />
+            '''
+            )))
+
+        content = Content()
+        a1 = zope.component.getMultiAdapter((content, Request(IR)))
+        self.assert_(isinstance(a1, A1))
+
+        class MyContent:
+            interface.implements(IContent)
+
+        self.assertRaises(ComponentLookupError, zope.component.getMultiAdapter,
+                          (MyContent(), Request(IR)))
+
+    def testInterfaceProtectedView(self):
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"
+                  permission="zope.Public"
+              allowed_interface="zope.component.testfiles.views.IV"
+                  />
+            '''
+            ))
+
+        v = ProxyFactory(zope.component.getMultiAdapter((Ob3(), Request(IV)),
+                                                        name='test'))
+        self.assertEqual(v.index(), 'V1 here')
+        self.assertRaises(Exception, getattr, v, 'action')
+
+    def testAttributeProtectedView(self):
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"
+                  permission="zope.Public"
+                  allowed_attributes="action"
+                  />
+            '''
+            ))
+
+        v = ProxyFactory(zope.component.getMultiAdapter((Ob3(), Request(IV)),
+                                                        name='test'))
+        self.assertEqual(v.action(), 'done')
+        self.assertRaises(Exception, getattr, v, 'index')
+
+    def testInterfaceAndAttributeProtectedView(self):
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"
+                  permission="zope.Public"
+                  allowed_attributes="action"
+              allowed_interface="zope.component.testfiles.views.IV"
+                  />
+            '''
+            ))
+
+        v = zope.component.getMultiAdapter((Ob3(), Request(IV)), name='test')
+        self.assertEqual(v.index(), 'V1 here')
+        self.assertEqual(v.action(), 'done')
+
+    def testDuplicatedInterfaceAndAttributeProtectedView(self):
+        xmlconfig(StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"
+                  permission="zope.Public"
+                  allowed_attributes="action index"
+              allowed_interface="zope.component.testfiles.views.IV"
+                  />
+            '''
+            ))
+
+        v = zope.component.getMultiAdapter((Ob3(), Request(IV)), name='test')
+        self.assertEqual(v.index(), 'V1 here')
+        self.assertEqual(v.action(), 'done')
+
+    def testIncompleteProtectedViewNoPermission(self):
+        self.assertRaises(
+            ConfigurationError,
+            xmlconfig,
+            StringIO(template %
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"
+                  allowed_attributes="action index"
+                  />
+            '''
+            ))
+
+    def testViewUndefinedPermission(self):
+        config = StringIO(template % (
+            '''
+            <view name="test"
+                  factory="zope.component.testfiles.views.V1"
+                  for="zope.component.testfiles.views.IC"
+                  type="zope.component.testfiles.views.IV"
+                  permission="zope.UndefinedPermission"
+                  allowed_attributes="action index"
+              allowed_interface="zope.component.testfiles.views.IV"
+                  />
+            '''
+            ))
+        self.assertRaises(ValueError, xmlconfig, config, testing=1)
+
+    def testResource(self):
+        ob = Ob3()
+        self.assertEqual(
+            zope.component.queryAdapter(Request(IV), name=u'test'), None)
+        xmlconfig(StringIO(template % (
+            '''
+            <resource name="test"
+                  factory="zope.component.testfiles.views.R1"
+                  type="zope.component.testfiles.views.IV"/>
+            '''
+            )))
+
+        self.assertEqual(
+            zope.component.queryAdapter(Request(IV), name=u'test').__class__,
+            R1)
+
+    def testResourceThatProvidesAnInterface(self):
+        ob = Ob3()
+        self.assertEqual(zope.component.queryAdapter(Request(IR), IV, u'test'),
+                         None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <resource
+                name="test"
+                factory="zope.component.testfiles.views.R1"
+                type="zope.component.testfiles.views.IR"
+                />
+            '''
+            ))
+
+        v = zope.component.queryAdapter(Request(IR), IV, name=u'test')
+        self.assertEqual(v, None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <resource
+                name="test"
+                factory="zope.component.testfiles.views.R1"
+                type="zope.component.testfiles.views.IR"
+                provides="zope.component.testfiles.views.IV"
+                />
+            '''
+            ))
+
+        v = zope.component.queryAdapter(Request(IR), IV, name=u'test')
+        self.assertEqual(v.__class__, R1)
+
+    def testUnnamedResourceThatProvidesAnInterface(self):
+        ob = Ob3()
+        self.assertEqual(zope.component.queryAdapter(Request(IR), IV), None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <resource
+                factory="zope.component.testfiles.views.R1"
+                type="zope.component.testfiles.views.IR"
+                />
+            '''
+            ))
+
+        v = zope.component.queryAdapter(Request(IR), IV)
+        self.assertEqual(v, None)
+
+        xmlconfig(StringIO(template %
+            '''
+            <resource
+                factory="zope.component.testfiles.views.R1"
+                type="zope.component.testfiles.views.IR"
+                provides="zope.component.testfiles.views.IV"
+                />
+            '''
+            ))
+
+        v = zope.component.queryAdapter(Request(IR), IV)
+        self.assertEqual(v.__class__, R1)
+
+    def testResourceUndefinedPermission(self):
+
+        config = StringIO(template % (
+            '''
+            <resource name="test"
+                  factory="zope.component.testfiles.views.R1"
+                  type="zope.component.testfiles.views.IV"
+                  permission="zope.UndefinedPermission"/>
+            '''
+            ))
+        self.assertRaises(ValueError, xmlconfig, config, testing=1)
+
 
 def setUpRegistryTests(tests):
     setUp()
@@ -1200,8 +1605,6 @@ def tearDownRegistryTests(tests):
 def clearZCML(test=None):
     tearDown()
     setUp()
-
-    from zope.configuration.xmlconfig import XMLConfig
     XMLConfig('meta.zcml', component)()
 
 def test_suite():
@@ -1213,6 +1616,7 @@ def test_suite():
 
     return unittest.TestSuite((
         doctest.DocTestSuite(setUp=setUp, tearDown=tearDown),
+        unittest.makeSuite(HookableTests),
         doctest.DocTestSuite('zope.component.interface',
                              setUp=setUp, tearDown=tearDown),
         doctest.DocTestSuite('zope.component.nexttesting'),
@@ -1230,6 +1634,7 @@ def test_suite():
         doctest.DocFileSuite('zcml.txt',checker=checker,
                              setUp=setUp, tearDown=tearDown),
         unittest.makeSuite(StandaloneTests),
+        unittest.makeSuite(ResourceViewTests),
         ))
 
 if __name__ == "__main__":
